@@ -413,6 +413,7 @@ class DatabaseWrapper(base.DatabaseWrapper):
                     '2013',  # Lost connection to MySQL server during query
             ):
                 atomic_blocks = copy.deepcopy(getattr(self, 'atomic_blocks', None))
+                run_on_commit = copy.deepcopy(getattr(self, 'run_on_commit', None))
                 autocommit = self.autocommit
                 closed_in_transaction = self.closed_in_transaction
                 commit_on_exit = self.commit_on_exit
@@ -448,12 +449,15 @@ class DatabaseWrapper(base.DatabaseWrapper):
                     time.sleep(self.reconnect_wait_time)
 
                 self.connect()
-                self.set_autocommit(False, force_begin_transaction_with_broken_autocommit=True)
+                self.connection.autocommit(False)
+                self.autocommit = False
                 self.needs_rollback = True
                 cursor = self.replay_history(history)
                 self.needs_rollback = needs_rollback
                 if hasattr(self, 'atomic_blocks'):
                     self.atomic_blocks = atomic_blocks
+                if hasattr(self, 'run_on_commit'):
+                    self.run_on_commit = run_on_commit
                 if autocommit:
                     self.connection.commit()
                     self.set_autocommit(autocommit)
